@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import mock
+from mock import patch
 from oslo_config import cfg
 
 from neutron.common import constants as n_const
@@ -33,6 +34,7 @@ def setup_arista_wrapper_config(value=''):
     cfg.CONF.set_override('eapi_host', value, "ml2_arista")
     cfg.CONF.set_override('eapi_username', value, "ml2_arista")
     cfg.CONF.set_override('sync_interval', 10, "ml2_arista")
+    cfg.CONF.set_override('conn_timeout', 20, "ml2_arista")
 
 
 def setup_valid_config():
@@ -101,7 +103,7 @@ class AristaProvisionedVlansStorageTestCase(testlib_api.SqlTestCase):
         host_id = 'ubuntu1'
 
         db_lib.remember_vm(vm_id, host_id, port_id, network_id, tenant_id)
-        db_lib.forget_vm(vm_id, host_id, port_id, network_id, tenant_id)
+        db_lib.forget_port(port_id, host_id)
         vm_provisioned = db_lib.is_vm_provisioned(vm_id, host_id, port_id,
                                                   network_id, tenant_id)
         self.assertFalse(vm_provisioned, 'The vm should be deleted')
@@ -162,16 +164,18 @@ class AristaProvisionedVlansStorageTestCase(testlib_api.SqlTestCase):
     def test_num_vm_is_valid(self):
         tenant_id = 'test'
         network_id = '123'
-        port_id = 456
+        port_id_base = 'port-id'
         host_id = 'ubuntu1'
 
         vm_to_remember = ['vm1', 'vm2', 'vm3']
         vm_to_forget = ['vm2', 'vm1']
 
         for vm in vm_to_remember:
+            port_id = port_id_base + vm
             db_lib.remember_vm(vm, host_id, port_id, network_id, tenant_id)
         for vm in vm_to_forget:
-            db_lib.forget_vm(vm, host_id, port_id, network_id, tenant_id)
+            port_id = port_id_base + vm
+            db_lib.forget_port(port_id, host_id)
 
         num_vms = len(db_lib.get_vms(tenant_id))
         expected = len(vm_to_remember) - len(vm_to_forget)
@@ -180,7 +184,7 @@ class AristaProvisionedVlansStorageTestCase(testlib_api.SqlTestCase):
                          'There should be %d records, '
                          'got %d records' % (expected, num_vms))
         # clean up afterwards
-        db_lib.forget_vm('vm3', host_id, port_id, network_id, tenant_id)
+        db_lib.forget_port(port_id, host_id)
 
     def test_get_network_list_returns_eos_compatible_data(self):
         tenant = u'test-1'
@@ -214,16 +218,32 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
         super(PositiveRPCWrapperValidConfigTestCase, self).setUp()
         setup_valid_config()
         self.drv = arista_ml2.AristaRPCWrapper()
+        self.drv._server_ip = "10.11.12.13"
         self.region = 'RegionOne'
-        self.drv._server = mock.MagicMock()
 
     def _get_exit_mode_cmds(self, modes):
         return ['exit'] * len(modes)
 
+    def _verify_send_eapi_request_calls(self, mock_send_eapi_req, cmds):
+        # Need this method as unfortunately, a direct comparison between
+        # Mock calls and expected call objects doesn't compare the list of
+        # commands that have been sent
+        for cmd in cmds:
+            found = False
+            for call in mock_send_eapi_req.mock_calls:
+                if cmd == call[2]['cmds']:
+                    found = True
+                    break
+            if not found:
+                assert (0,
+                        "Failed to find command '%s' in %s" % (
+                            cmd, mock_send_eapi_req.mock_calls))
+
     def test_no_exception_on_correct_configuration(self):
         self.assertIsNotNone(self.drv)
 
-    def test_plug_host_into_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_plug_host_into_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         vm_id = 'vm-1'
         port_id = 123
@@ -233,15 +253,16 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         self.drv.plug_host_into_network(vm_id, host, port_id,
                                         network_id, tenant_id, port_name)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'vm id vm-1 hostid host',
                 'port id 123 name "123-port" network-id net-id',
                 ]
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_plug_dhcp_port_into_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_plug_dhcp_port_into_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         vm_id = 'vm-1'
         port_id = 123
@@ -251,15 +272,16 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         self.drv.plug_dhcp_port_into_network(vm_id, host, port_id,
                                              network_id, tenant_id, port_name)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'network id net-id',
                 'dhcp id vm-1 hostid host port-id 123 name "123-port"',
                 ]
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_unplug_host_from_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_unplug_host_from_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         vm_id = 'vm-1'
         port_id = 123
@@ -267,14 +289,16 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
         host = 'host'
         self.drv.unplug_host_from_network(vm_id, host, port_id,
                                           network_id, tenant_id)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'vm id vm-1 hostid host',
                 'no port id 123',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_unplug_dhcp_port_from_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_unplug_dhcp_port_from_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         vm_id = 'vm-1'
         port_id = 123
@@ -283,15 +307,16 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         self.drv.unplug_dhcp_port_from_network(vm_id, host, port_id,
                                                network_id, tenant_id)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'network id net-id',
                 'no dhcp id vm-1 port-id 123',
                 ]
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_create_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         network = {
             'network_id': 'net-id',
@@ -300,15 +325,18 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
             'shared': False,
             }
         self.drv.create_network(tenant_id, network)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'network id net-id name "net-name"',
                 'segment 1 type vlan id 123',
                 'no shared',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
 
-    def test_create_shared_network(self):
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
+
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_shared_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         network = {
             'network_id': 'net-id',
@@ -316,15 +344,17 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
             'segmentation_id': 123,
             'shared': True}
         self.drv.create_network(tenant_id, network)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'network id net-id name "net-name"',
                 'segment 1 type vlan id 123',
                 'shared',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_create_network_bulk(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_network_bulk(self, mock_send_eapi_req):
         tenant_id = 'ten-2'
         num_networks = 10
         networks = [{
@@ -336,31 +366,34 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
         ]
 
         self.drv.create_network_bulk(tenant_id, networks)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
                 'region RegionOne',
                 'tenant ten-2']
         for net_id in range(1, num_networks):
-            cmds.append('network id net-id-%d name "net-name-%d"' %
+            cmd2.append('network id net-id-%d name "net-name-%d"' %
                         (net_id, net_id))
-            cmds.append('segment 1 type vlan id %d' % net_id)
-            cmds.append('shared')
+            cmd2.append('segment 1 type vlan id %d' % net_id)
+            cmd2.append('shared')
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_delete_network(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_network(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         network_id = 'net-id'
         self.drv.delete_network(tenant_id, network_id)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'no network id net-id',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_network_bulk(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_network_bulk(self, mock_send_eapi_req):
         tenant_id = 'ten-2'
         num_networks = 10
         networks = [{
@@ -371,34 +404,38 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         networks = ['net-id-%d' % net_id for net_id in range(1, num_networks)]
         self.drv.delete_network_bulk(tenant_id, networks)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
                 'region RegionOne',
                 'tenant ten-2']
         for net_id in range(1, num_networks):
-            cmds.append('no network id net-id-%d' % net_id)
+            cmd2.append('no network id net-id-%d' % net_id)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_delete_vm(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_vm(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         vm_id = 'vm-id'
         self.drv.delete_vm(tenant_id, vm_id)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne',
                 'tenant ten-1', 'no vm id vm-id',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_vm_bulk(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_vm_bulk(self, mock_send_eapi_req):
         tenant_id = 'ten-2'
         num_vms = 10
         vm_ids = ['vm-id-%d' % vm_id for vm_id in range(1, num_vms)]
         self.drv.delete_vm_bulk(tenant_id, vm_ids)
 
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -406,41 +443,47 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
                 'tenant ten-2']
 
         for vm_id in range(1, num_vms):
-            cmds.append('no vm id vm-id-%d' % vm_id)
+            cmd2.append('no vm id vm-id-%d' % vm_id)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
-
-    def test_create_vm_port_bulk(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_port_bulk(self, mock_send_eapi_req):
         tenant_id = 'ten-3'
-        num_vms = 10
-        num_ports_per_vm = 2
+        num_devices = 10
+        num_ports_per_device = 2
 
-        vms = dict(
-            ('vm-id-%d' % vm_id, {
-                'vmId': 'vm-id-%d' % vm_id,
-                'host': 'host_%d' % vm_id,
-            }
-            ) for vm_id in range(1, num_vms)
-        )
+        ports = {}
+        device_count = 0
+        for device_id in range(1, num_devices):
+            device_count += 1
+            for port_id in range(1, num_ports_per_device):
+                port_id = 'port-id-%d-%d' % (device_id, port_id)
+                ports[port_id] = {
+                    'deviceId': 'dev-id-%d' % device_id,
+                    'hosts': ['host_%d' % (device_count)]
+                }
 
-        devices = [n_const.DEVICE_OWNER_DHCP, 'compute']
-        vm_port_list = []
+        devices = [n_const.DEVICE_OWNER_DHCP, 'compute',
+                   n_const.DEVICE_OWNER_DVR_INTERFACE]
+        port_list = []
 
         net_count = 1
-        for vm_id in range(1, num_vms):
-            for port_id in range(1, num_ports_per_vm):
+        for device_id in range(1, num_devices):
+            for port_id in range(1, num_ports_per_device):
                 port = {
-                    'id': 'port-id-%d-%d' % (vm_id, port_id),
-                    'device_id': 'vm-id-%d' % vm_id,
-                    'device_owner': devices[(vm_id + port_id) % 2],
+                    'id': 'port-id-%d-%d' % (device_id, port_id),
+                    'device_id': 'dev-id-%d' % device_id,
+                    'device_owner': devices[(device_id + port_id) % 3],
                     'network_id': 'network-id-%d' % net_count,
-                    'name': 'port-%d-%d' % (vm_id, port_id)
+                    'name': 'port-%d-%d' % (device_id, port_id)
                 }
-                vm_port_list.append(port)
+                port_list.append(port)
                 net_count += 1
 
-        self.drv.create_vm_port_bulk(tenant_id, vm_port_list, vms)
-        cmds = ['enable',
+        self.drv.cli_commands[arista_ml2.CMD_INSTANCE] = 'instance'
+        self.drv.create_port_bulk(tenant_id, port_list, ports)
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -448,47 +491,55 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
                 'tenant ten-3']
 
         net_count = 1
-        for vm_count in range(1, num_vms):
+        for vm_count in range(1, num_devices):
             host = 'host_%s' % vm_count
-            for port_count in range(1, num_ports_per_vm):
-                vm_id = 'vm-id-%d' % vm_count
+            for port_count in range(1, num_ports_per_device):
+                device_id = 'dev-id-%d' % vm_count
                 device_owner = devices[(vm_count + port_count) % 2]
                 port_name = '"port-%d-%d"' % (vm_count, port_count)
                 network_id = 'network-id-%d' % net_count
                 port_id = 'port-id-%d-%d' % (vm_count, port_count)
                 if device_owner == 'network:dhcp':
-                    cmds.append('network id %s' % network_id)
-                    cmds.append('dhcp id %s hostid %s port-id %s name %s' % (
-                                vm_id, host, port_id, port_name))
+                    cmd2.append('network id %s' % network_id)
+                    cmd2.append('dhcp id %s hostid %s port-id %s name %s' % (
+                                device_id, host, port_id, port_name))
                 elif device_owner == 'compute':
-                    cmds.append('vm id %s hostid %s' % (vm_id, host))
-                    cmds.append('port id %s name %s network-id %s' % (
+                    cmd2.append('vm id %s hostid %s' % (device_id, host))
+                    cmd2.append('port id %s name %s network-id %s' % (
                                 port_id, port_name, network_id))
+                elif device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
+                    cmd2.append('instance id %s type router' % device_id)
+                    cmd2.append('port id %s network-id %s hostid %s' % (
+                                port_id, network_id, host))
                 net_count += 1
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_tenant(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_tenant(self, mock_send_eapi_req):
         tenant_id = 'ten-1'
         self.drv.delete_tenant(tenant_id)
-        cmds = ['enable', 'configure', 'cvx', 'service openstack',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable', 'configure', 'cvx', 'service openstack',
                 'region RegionOne', 'no tenant ten-1',
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_tenant_bulk(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_tenant_bulk(self, mock_send_eapi_req):
         num_tenants = 10
         tenant_list = ['ten-%d' % t_id for t_id in range(1, num_tenants)]
         self.drv.delete_tenant_bulk(tenant_list)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
                 'region RegionOne']
         for ten_id in range(1, num_tenants):
-            cmds.append('no tenant ten-%d' % ten_id)
+            cmd2.append('no tenant ten-%d' % ten_id)
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
     def test_get_network_info_returns_none_when_no_such_net(self):
         expected = []
@@ -514,24 +565,36 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
         self.assertEqual(net_info, valid_net_info,
                          ('Must return network info for a valid net'))
 
-    def test_check_cli_commands(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_check_cli_commands(self, mock_send_eapi_req):
         self.drv._get_random_name = mock.MagicMock()
         self.drv._get_random_name.return_value = 'RegionOne'
+
         self.drv.check_cli_commands()
+
         timestamp_cmd = ['show openstack config region RegionOne timestamp']
         sync_lock_cmd = ['enable', 'configure', 'cvx', 'service openstack',
                          'region RegionOne', 'sync lock clientid requestid',
                          'exit', 'region RegionOne sync', 'sync end', 'exit']
         delete_region_cmd = ['enable', 'configure', 'cvx', 'service openstack',
                              'no region RegionOne']
-        expected_calls = [
-            mock.call.runCmds(cmds=timestamp_cmd, version=1),
-            mock.call.runCmds(cmds=sync_lock_cmd, version=1),
-            mock.call.runCmds(cmds=delete_region_cmd, version=1),
-        ]
-        assert self.drv._server.mock_calls == expected_calls
+        instance_command = ['enable', 'configure', 'cvx', 'service openstack',
+                            'region RegionOne', 'tenant t1',
+                            'instance id i1 type router']
+        cmds = [timestamp_cmd, sync_lock_cmd, delete_region_cmd,
+                instance_command, delete_region_cmd]
 
-    def test_register_with_eos(self):
+        for cmd in cmds:
+            found = False
+            for call in mock_send_eapi_req.mock_calls:
+                if cmd == call[2]['cmds']:
+                    found = True
+                    break
+            if not found:
+                assert 0, "Failed to find a command that should've been called"
+
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_register_with_eos(self, mock_send_eapi_req):
         self.drv.register_with_eos()
         auth = fake_keystone_info_class()
         keystone_url = '%s://%s:%s/v2.0/' % (auth.auth_protocol,
@@ -542,7 +605,8 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
                     auth.admin_user,
                     auth.admin_password,
                     auth.admin_tenant_name))
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -550,14 +614,15 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
                 auth_cmd,
                 'sync interval %d' % cfg.CONF.ml2_arista.sync_interval,
                 ]
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
     def _enable_sync_cmds(self):
         self.drv.cli_commands[
             arista_ml2.CMD_REGION_SYNC] = 'region RegionOne sync'
         self.drv.cli_commands[arista_ml2.CMD_SYNC_HEARTBEAT] = 'sync heartbeat'
 
-    def test_create_network_bulk_during_sync(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_network_bulk_during_sync(self, mock_send_eapi_req):
         self._enable_sync_cmds()
         tenant_id = 'ten-10'
         num_networks = 101
@@ -570,7 +635,8 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
         ]
 
         self.drv.create_network_bulk(tenant_id, networks, sync=True)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -579,30 +645,32 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         # Send 100 create network commands
         for net_id in range(1, 101):
-            cmds.append('network id net-id-%d name "net-name-%d"' %
+            cmd2.append('network id net-id-%d name "net-name-%d"' %
                         (net_id, net_id))
-            cmds.append('segment 1 type vlan id %d' % net_id)
-            cmds.append('shared')
+            cmd2.append('segment 1 type vlan id %d' % net_id)
+            cmd2.append('shared')
 
         # Send heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
         # Send the remaining network
-        cmds.append('network id net-id-101 name "net-name-101"')
-        cmds.append('segment 1 type vlan id 101')
-        cmds.append('shared')
+        cmd2.append('network id net-id-101 name "net-name-101"')
+        cmd2.append('segment 1 type vlan id 101')
+        cmd2.append('shared')
         # Send the final heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_network_bulk_during_sync(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_network_bulk_during_sync(self, mock_send_eapi_req):
         self._enable_sync_cmds()
         tenant_id = 'ten-10'
         num_networks = 101
         networks = ['nid-%d' % net_id for net_id in range(1, num_networks + 1)]
 
         self.drv.delete_network_bulk(tenant_id, networks, sync=True)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -611,25 +679,27 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
 
         # Send 100 create network commands
         for net_id in range(1, 101):
-            cmds.append('no network id nid-%d' % (net_id))
+            cmd2.append('no network id nid-%d' % (net_id))
 
         # Send heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
         # Send the remaining network
-        cmds.append('no network id nid-101')
+        cmd2.append('no network id nid-101')
         # Send the final heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_vm_bulk_during_sync(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_vm_bulk_during_sync(self, mock_send_eapi_req):
         self._enable_sync_cmds()
         tenant_id = 'ten-2'
         num_vms = 101
         vm_ids = ['vm-id-%d' % vm_id for vm_id in range(1, num_vms + 1)]
         self.drv.delete_vm_bulk(tenant_id, vm_ids, sync=True)
 
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
@@ -637,82 +707,85 @@ class PositiveRPCWrapperValidConfigTestCase(base.BaseTestCase):
                 'tenant ten-2']
 
         for vm_id in range(1, 101):
-            cmds.append('no vm id vm-id-%d' % vm_id)
+            cmd2.append('no vm id vm-id-%d' % vm_id)
 
         # Send heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
         # Send the remaining vm
-        cmds.append('no vm id vm-id-101')
+        cmd2.append('no vm id vm-id-101')
         # Send the final heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_create_vm_port_bulk_during_sync(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_create_port_bulk_during_sync(self, mock_send_eapi_req):
         self._enable_sync_cmds()
         tenant_id = 'ten-3'
-        num_vms = 101
+        num_ports = 101
 
-        vms = dict(
-            ('vm-id-%d' % vm_id, {
-                'vmId': 'vm-id-%d' % vm_id,
-                'host': 'host_%d' % vm_id,
+        ports = dict(
+            ('pid-%d' % id, {
+                'deviceId': 'vm-id-%d' % id,
+                'hosts': ['host_%d' % id],
             }
-            ) for vm_id in range(1, num_vms + 1)
+            ) for id in range(1, num_ports + 1)
         )
 
-        vm_port_list = []
+        port_list = []
 
-        for vm_id in range(1, num_vms + 1):
+        for id in range(1, num_ports + 1):
             port = {
-                'id': 'pid-%d' % vm_id,
-                'device_id': 'vm-id-%d' % vm_id,
+                'id': 'pid-%d' % id,
+                'device_id': 'vm-id-%d' % id,
                 'device_owner': 'compute',
-                'network_id': 'nid-%d' % vm_id,
-                'name': 'pname-%d' % vm_id,
+                'network_id': 'nid-%d' % id,
+                'name': 'pname-%d' % id,
             }
-            vm_port_list.append(port)
+            port_list.append(port)
 
-        self.drv.create_vm_port_bulk(tenant_id, vm_port_list, vms, sync=True)
-        cmds = ['enable',
+        self.drv.create_port_bulk(tenant_id, port_list, ports, sync=True)
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
                 'region RegionOne sync',
                 'tenant ten-3']
 
-        for vm_count in range(1, 101):
-            vm_id = 'vm-id-%d' % vm_count
-            cmds.append('vm id vm-id-%d hostid host_%d' % (vm_count, vm_count))
-            cmds.append('port id pid-%d name "pname-%d" network-id nid-%s' % (
-                        vm_count, vm_count, vm_count))
+        for id in range(1, 101):
+            cmd2.append('vm id vm-id-%d hostid host_%d' % (id, id))
+            cmd2.append('port id pid-%d name "pname-%d" network-id nid-%s' % (
+                        id, id, id))
 
         # Send heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
         # Send the remaining vm port
-        cmds.append('vm id vm-id-101 hostid host_101')
-        cmds.append('port id pid-101 name "pname-101" network-id nid-101')
+        cmd2.append('vm id vm-id-101 hostid host_101')
+        cmd2.append('port id pid-101 name "pname-101" network-id nid-101')
         # Send the final heartbeat
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
-    def test_delete_tenant_bulk_during_sync(self):
+    @patch('networking_arista.ml2.arista_ml2.AristaRPCWrapper._send_eapi_req')
+    def test_delete_tenant_bulk_during_sync(self, mock_send_eapi_req):
         self._enable_sync_cmds()
         num_tenants = 101
         tenant_list = ['ten-%d' % t_id for t_id in range(1, num_tenants + 1)]
         self.drv.delete_tenant_bulk(tenant_list, sync=True)
-        cmds = ['enable',
+        cmd1 = ['show openstack agent uuid']
+        cmd2 = ['enable',
                 'configure',
                 'cvx',
                 'service openstack',
                 'region RegionOne sync']
         for ten_id in range(1, num_tenants + 1):
-            cmds.append('no tenant ten-%d' % ten_id)
+            cmd2.append('no tenant ten-%d' % ten_id)
 
-        cmds.append('sync heartbeat')
+        cmd2.append('sync heartbeat')
 
-        self.drv._server.runCmds.assert_called_once_with(version=1, cmds=cmds)
+        self._verify_send_eapi_request_calls(mock_send_eapi_req, [cmd1, cmd2])
 
 
 class AristaRPCWrapperInvalidConfigTestCase(base.BaseTestCase):
@@ -1015,6 +1088,7 @@ class SyncServiceTest(testlib_api.SqlTestCase):
             mock.call.get_region_updated_time(),
             mock.call.sync_start(),
             mock.call.register_with_eos(sync=True),
+            mock.call.check_cli_commands(),
             mock.call.get_tenants(),
             mock.call.create_network_bulk(
                 tenant_id,
@@ -1098,8 +1172,10 @@ class SyncServiceTest(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.get_region_updated_time(),
+            mock.call.get_region_updated_time().__nonzero__(),
             mock.call.sync_start(),
             mock.call.register_with_eos(sync=True),
+            mock.call.check_cli_commands(),
             mock.call.get_tenants(),
             mock.call.create_network_bulk(
                 tenant_2_id,
@@ -1152,16 +1228,11 @@ class SyncServiceTest(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.get_region_updated_time(),
+            mock.call.get_region_updated_time().__nonzero__(),
             mock.call.sync_start(),
             mock.call.register_with_eos(sync=True),
+            mock.call.check_cli_commands(),
             mock.call.get_tenants(),
-            mock.call.create_network_bulk(
-                tenant_2_id,
-                [{'network_id': tenant_2_net_1_id,
-                  'segmentation_id': tenant_2_net_1_seg_id,
-                  'network_name': '',
-                  'shared': False}],
-                sync=True),
             mock.call.create_network_bulk(
                 tenant_1_id,
                 [{'network_id': tenant_1_net_1_id,
@@ -1169,6 +1240,13 @@ class SyncServiceTest(testlib_api.SqlTestCase):
                   'network_name': '',
                   'shared': False}],
                 sync=True),
+            mock.call.create_network_bulk(
+                tenant_2_id,
+                [{'network_id': tenant_2_net_1_id,
+                  'segmentation_id': tenant_2_net_1_seg_id,
+                  'network_name': '',
+                  'shared': False}],
+                sync=True),
             mock.call.sync_end(),
             mock.call.get_region_updated_time()
         ]
@@ -1176,7 +1254,7 @@ class SyncServiceTest(testlib_api.SqlTestCase):
         # The create_network_bulk() can be called in different order. So split
         # it up. The first part checks if the initial set of methods are
         # invoked.
-        self.assertTrue(self.rpc.mock_calls[:4] == expected_calls[:4],
+        self.assertTrue(self.rpc.mock_calls[:5] == expected_calls[:5],
                         "Seen: %s\nExpected: %s" % (
                             self.rpc.mock_calls,
                             expected_calls,
@@ -1184,7 +1262,7 @@ class SyncServiceTest(testlib_api.SqlTestCase):
                         )
         # Check if tenant 1 networks are created. It must be one of the two
         # methods.
-        self.assertTrue(self.rpc.mock_calls[4] in expected_calls[4:6],
+        self.assertTrue(self.rpc.mock_calls[5] in expected_calls[5:7],
                         "Seen: %s\nExpected: %s" % (
                             self.rpc.mock_calls,
                             expected_calls,
@@ -1192,14 +1270,14 @@ class SyncServiceTest(testlib_api.SqlTestCase):
                         )
         # Check if tenant 2 networks are created. It must be one of the two
         # methods.
-        self.assertTrue(self.rpc.mock_calls[5] in expected_calls[4:6],
+        self.assertTrue(self.rpc.mock_calls[6] in expected_calls[6:8],
                         "Seen: %s\nExpected: %s" % (
-                            self.rpc.mock_calls,
-                            expected_calls,
+                            self.rpc.mock_calls[6],
+                            expected_calls[6:8],
                             )
                         )
         # Check if the sync end methods are invoked.
-        self.assertTrue(self.rpc.mock_calls[6:8] == expected_calls[6:8],
+        self.assertTrue(self.rpc.mock_calls[8:10] == expected_calls[8:10],
                         "Seen: %s\nExpected: %s" % (
                             self.rpc.mock_calls,
                             expected_calls,
@@ -1218,6 +1296,7 @@ class fake_keystone_info_class(object):
     Arista Driver expects Keystone auth info. This fake information
     is for testing only
     """
+    auth_uri = False
     auth_protocol = 'abc'
     auth_host = 'host'
     auth_port = 5000
